@@ -147,10 +147,13 @@ def _multiply_seq_by_int(self, opnode, other, context):
 def _filter_uninferable_nodes(elts, context):
     for elt in elts:
         if elt is util.Uninferable:
-            yield elt
+            yield nodes.Unknown()
         else:
             for inferred in elt.infer(context):
-                yield inferred
+                if inferred is not util.Uninferable:
+                    yield inferred
+                else:
+                    yield nodes.Unknown()
 
 
 @decorators.yes_if_nothing_inferred
@@ -242,6 +245,10 @@ def _resolve_looppart(parts, asspath, context):
 
 @decorators.raise_if_nothing_inferred
 def for_assigned_stmts(self, node=None, context=None, asspath=None):
+    if isinstance(self, nodes.AsyncFor) or getattr(self, 'is_async', False):
+        # Skip inferring of async code for now
+        raise StopIteration(dict(node=self, unknown=node,
+                                 assign_path=asspath, context=context))
     if asspath is None:
         for lst in self.iter.infer(context):
             if isinstance(lst, (nodes.Tuple, nodes.List)):
@@ -356,7 +363,16 @@ def assign_assigned_stmts(self, node=None, context=None, asspath=None):
     raise StopIteration(dict(node=self, unknown=node,
                              assign_path=asspath, context=context))
 
+
+def assign_annassigned_stmts(self, node=None, context=None, asspath=None):
+    for inferred in assign_assigned_stmts(self, node, context, asspath):
+        if inferred is None:
+            yield util.Uninferable
+        else:
+            yield inferred
+
 nodes.Assign.assigned_stmts = assign_assigned_stmts
+nodes.AnnAssign.assigned_stmts = assign_annassigned_stmts
 nodes.AugAssign.assigned_stmts = assign_assigned_stmts
 
 
