@@ -57,6 +57,7 @@ from astroid import bases
 from astroid import builder
 from astroid import nodes
 from astroid import util
+from astroid import test_utils
 import astroid
 
 
@@ -124,11 +125,11 @@ class NamedTupleTest(unittest.TestCase):
         result = __(six.moves.urllib.parse.urlparse('gopher://'))
         """)
         instance = next(result.infer())
-        self.assertEqual(len(instance.getattr('scheme')), 1)
-        self.assertEqual(len(instance.getattr('port')), 1)
+        self.assertGreaterEqual(len(instance.getattr('scheme')), 1)
+        self.assertGreaterEqual(len(instance.getattr('port')), 1)
         with self.assertRaises(astroid.AttributeInferenceError):
             instance.getattr('foo')
-        self.assertEqual(len(instance.getattr('geturl')), 1)
+        self.assertGreaterEqual(len(instance.getattr('geturl')), 1)
         self.assertEqual(instance.name, 'ParseResult')
 
     def test_namedtuple_instance_attrs(self):
@@ -150,6 +151,61 @@ class NamedTupleTest(unittest.TestCase):
         inferred = next(node.infer())
         self.assertIs(util.Uninferable, inferred)
 
+    def test_namedtuple_access_class_fields(self):
+        node = builder.extract_node("""
+        from collections import namedtuple
+        Tuple = namedtuple("Tuple", "field other")
+        Tuple #@
+        """)
+        inferred = next(node.infer())
+        self.assertIn('field', inferred.locals)
+        self.assertIn('other', inferred.locals)
+
+    def test_namedtuple_rename_keywords(self):
+        node = builder.extract_node("""
+        from collections import namedtuple
+        Tuple = namedtuple("Tuple", "abc def", rename=True)
+        Tuple #@
+        """)
+        inferred = next(node.infer())
+        self.assertIn('abc', inferred.locals)
+        self.assertIn('_1', inferred.locals)
+
+    def test_namedtuple_rename_duplicates(self):
+        node = builder.extract_node("""
+        from collections import namedtuple
+        Tuple = namedtuple("Tuple", "abc abc abc", rename=True)
+        Tuple #@
+        """)
+        inferred = next(node.infer())
+        self.assertIn('abc', inferred.locals)
+        self.assertIn('_1', inferred.locals)
+        self.assertIn('_2', inferred.locals)
+
+    def test_namedtuple_rename_uninferable(self):
+        node = builder.extract_node("""
+        from collections import namedtuple
+        Tuple = namedtuple("Tuple", "a b c", rename=UNINFERABLE)
+        Tuple #@
+        """)
+        inferred = next(node.infer())
+        self.assertIn('a', inferred.locals)
+        self.assertIn('b', inferred.locals)
+        self.assertIn('c', inferred.locals)
+
+
+class DefaultDictTest(unittest.TestCase):
+
+    def test_1(self):
+        node = builder.extract_node('''
+        from collections import defaultdict
+
+        X = defaultdict(int)
+        X[0]
+        ''')
+        inferred = next(node.infer())
+        self.assertIs(util.Uninferable, inferred)
+
 
 class ModuleExtenderTest(unittest.TestCase):
     def testExtensionModules(self):
@@ -526,5 +582,104 @@ class IOBrainTest(unittest.TestCase):
             self.assertEqual(raw.name, 'FileIO')
 
 
+@test_utils.require_version('3.6')
+class TypingBrain(unittest.TestCase):
+
+    def test_namedtuple_base(self):
+        klass = builder.extract_node("""
+        from typing import NamedTuple
+
+        class X(NamedTuple("X", [("a", int), ("b", str), ("c", bytes)])):
+           pass
+        """)
+        self.assertEqual(
+            [anc.name for anc in klass.ancestors()],
+            ['X', 'tuple', 'object'])
+        for anc in klass.ancestors():
+            self.assertFalse(anc.parent is None)
+
+    def test_namedtuple_inference(self):
+        klass = builder.extract_node("""
+        from typing import NamedTuple
+
+        class X(NamedTuple("X", [("a", int), ("b", str), ("c", bytes)])):
+           pass
+        """)
+        base = next(base for base in klass.ancestors()
+                    if base.name == 'X')
+        self.assertSetEqual({"a", "b", "c"}, set(base.instance_attrs))
+
+    def test_namedtuple_inference_nonliteral(self):
+        # Note: NamedTuples in mypy only work with literals.
+        klass = builder.extract_node("""
+        from typing import NamedTuple
+
+        name = "X"
+        fields = [("a", int), ("b", str), ("c", bytes)]
+        NamedTuple(name, fields)
+        """)
+        inferred = next(klass.infer())
+        self.assertIsInstance(inferred, astroid.Instance)
+        self.assertEqual(inferred.qname(), "typing.NamedTuple")
+
+    def test_namedtuple_instance_attrs(self):
+        result = builder.extract_node('''
+        from typing import NamedTuple
+        NamedTuple("A", [("a", int), ("b", str), ("c", bytes)])(1, 2, 3) #@
+        ''')
+        inferred = next(result.infer())
+        for name, attr in inferred.instance_attrs.items():
+            self.assertEqual(attr[0].attrname, name)
+
+    def test_namedtuple_simple(self):
+        result = builder.extract_node('''
+        from typing import NamedTuple
+        NamedTuple("A", [("a", int), ("b", str), ("c", bytes)])
+        ''')
+        inferred = next(result.infer())
+        self.assertIsInstance(inferred, nodes.ClassDef)
+        self.assertSetEqual({"a", "b", "c"}, set(inferred.instance_attrs))
+
+    def test_namedtuple_few_args(self):
+        result = builder.extract_node('''
+        from typing import NamedTuple
+        NamedTuple("A")
+        ''')
+        inferred = next(result.infer())
+        self.assertIsInstance(inferred, astroid.Instance)
+        self.assertEqual(inferred.qname(), "typing.NamedTuple")
+
+    def test_namedtuple_few_fields(self):
+        result = builder.extract_node('''
+        from typing import NamedTuple
+        NamedTuple("A", [("a",), ("b", str), ("c", bytes)])
+        ''')
+        inferred = next(result.infer())
+        self.assertIsInstance(inferred, astroid.Instance)
+        self.assertEqual(inferred.qname(), "typing.NamedTuple")
+
+    def test_namedtuple_class_form(self):
+        result = builder.extract_node('''
+        from typing import NamedTuple
+
+        class Example(NamedTuple):
+            mything: int
+
+        Example(mything=1)
+        ''')
+        inferred = next(result.infer())
+        self.assertIsInstance(inferred, astroid.Instance)
+
+
+class ReBrainTest(unittest.TestCase):
+    def test_regex_flags(self):
+        import re
+        names = [name for name in dir(re) if name.isupper()]
+        re_ast = MANAGER.ast_from_module_name('re')
+        for name in names:
+            self.assertIn(name, re_ast)
+            self.assertEqual(next(re_ast[name].infer()).value, getattr(re, name))
+
+
 if __name__ == '__main__':
     unittest.main()
